
struct vsInput
{
    float4 posObject : POSITION;
    float3 normal : NORMAL;	
	float4 uv: TEXCOORD0;
};

struct psInput
{
    float4 posObject : SV_Position;
    float3 normal : NORMAL;		
	float4 uv: TEXCOORD0;	
};

struct vsInputTextured
{
    float4 posObject : POSITION;
	float4 uv: TEXCOORD0;
    float3 normal : NORMAL;		
	
};

struct psInputTextured
{
    float4 posObject : SV_Position;
    float4 uv: TEXCOORD0;
	float4 color: COLOR;
		
    float3 normal : NORMAL;			
};

Texture2D inputTexture <string uiname="Texture";>;

SamplerState linearSampler <string uiname="Sampler State";>
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Clamp;
    AddressV = Clamp;
};

cbuffer cbPerDraw : register(b0)
{
	float4x4 tVP : VIEWPROJECTION;
	float4x4 tV: VIEW;
	float4x4 tV1 <string uiname="View1 Transform";>;	
	float4x4 tV2 <string uiname="View2 Transform";>;			
	float4x4 tV3 <string uiname="View3 Transform";>;			
};

cbuffer cbPerObj : register( b1 )
{
	float4x4 tW : WORLD;
	float4x4 tWIT: WORLDINVERSETRANSPOSE;	
	float Alpha <float uimin=0.0; float uimax=1.0;> = 1; 
	float4 cAmb <bool color=true;String uiname="Color";> = { 1.0f,1.0f,1.0f,1.0f };
	float4 cAmb1 <bool color=true;String uiname="Color Proj1";> = { 1.0f,1.0f,1.0f,1.0f };	
	float4 cAmb2 <bool color=true;String uiname="Color Proj2";> = { 1.0f,1.0f,1.0f,1.0f };		
	float4 cAmb3 <bool color=true;String uiname="Color Proj3";> = { 1.0f,1.0f,1.0f,1.0f };			
	float4x4 tColor <string uiname="Color Transform";>;
};

cbuffer cbTextureData : register(b2)
{
	float4x4 tTex <string uiname="Texture Transform"; bool uvspace=true; >;
};


psInputTextured VS_Textured(vsInputTextured input)
{
	psInputTextured output;
	output.posObject = (input.posObject);
	output.normal = (input.normal);	
	output.color = float4(1,1,1,1);	
	output.uv = mul(input.uv, tTex);
	
	return output;
}
[maxvertexcount(3)]
void GS_TexturedMesh(triangle psInputTextured input[3], inout TriangleStream<psInputTextured> gsout)
{
	psInputTextured o;
	
	float3 p1 = input[0].posObject.xyz;
	float3 p2 = input[1].posObject.xyz;
	float3 p3 = input[2].posObject.xyz;
	
	
	float3 faceEdgeA = p2 - p1;
    float3 faceEdgeB = p1 - p3;
    float3 norm = cross(faceEdgeB, faceEdgeA);	
	norm = normalize(norm);
/*
	float3 projNorm1 = normalize(mul(mul(norm, (float3x3)tWIT),(float3x3)tV1).xyz);
	float3 projNorm2 = normalize(mul(mul(norm, (float3x3)tWIT),(float3x3)tV2).xyz);		
	float3 projNorm3 = normalize(mul(mul(norm, (float3x3)tWIT),(float3x3)tV3).xyz);	
	
	float dot1 = dot(projNorm1,float3(0,0,-1));
	float dot2 = dot(projNorm2,float3(0,0,-1));	
	float dot3 = dot(projNorm3,float3(0,0,-1));	
	float4 color = float4(1,1,1,1);
	if ( (dot1 > dot2) && (dot1 > dot3) )
	{
		color = cAmb1;
	}
	if ( (dot2 > dot1) && (dot2 > dot3) )
	{
		color = cAmb2;
	}

	if ( (dot3 > dot1) && (dot3 > dot2) )
	{
		color = cAmb3;
	}
*/
	
	for (int i = 0; i <3; i ++) 
	{
			float3 projNorm1 = normalize(mul(mul(input[i].normal, (float3x3)tWIT),(float3x3)tV1).xyz);
	float3 projNorm2 = normalize(mul(mul(input[i].normal, (float3x3)tWIT),(float3x3)tV2).xyz);		
	float3 projNorm3 = normalize(mul(mul(input[i].normal, (float3x3)tWIT),(float3x3)tV3).xyz);	
	
	float dot1 = dot(projNorm1,float3(0,0,-1));
	float dot2 = dot(projNorm2,float3(0,0,-1));	
	float dot3 = dot(projNorm3,float3(0,0,-1));	
	float4 color = float4(1,1,1,1);
	/*
	if ( (dot1 > dot2) && (dot1 > dot3) )
	{
		color = cAmb1;
	}
	if ( (dot2 > dot1) && (dot2 > dot3) )
	{
		color = cAmb2;
	}

	if ( (dot3 > dot1) && (dot3 > dot2) )
	{
		color = cAmb3;
	}
	*/
	if ((projNorm1.z < projNorm2.z) && (projNorm1.z < projNorm3.z) )
	{
		color = cAmb1;
	}
	if ((projNorm2.z < projNorm1.z) && (projNorm2.z < projNorm3.z) )
	{
		color = cAmb2;
	}

	if ((projNorm3.z < projNorm2.z) && (projNorm3.z < projNorm1.z) )
	{
		color = cAmb3;
	}
		
		
		o.posObject =  mul(mul(float4(input[i].posObject.xyz,1),tW),tVP);
		o.uv = input[i].uv;
		o.normal = normalize(mul(mul(input[i].normal, (float3x3)tWIT),(float3x3)tV).xyz);	
		o.color = color;			
		gsout.Append(o);		
	}
	
	gsout.RestartStrip();
}



float4 PS_Textured(psInputTextured input): SV_Target
{
    float4 col = inputTexture.Sample(linearSampler,input.uv.xy) * cAmb * input.color;
	col.a *= Alpha;
    return col;
}

technique11 Constant
{
	pass P0
	{
		SetVertexShader( CompileShader( vs_4_0, VS_Textured() ) );
		SetGeometryShader( CompileShader( gs_4_0, GS_TexturedMesh() ) );		
		SetPixelShader( CompileShader( ps_4_0, PS_Textured() ) );
	}
}




